package com.scbfkj.uni.api;

import com.google.common.util.concurrent.RateLimiter;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.RequestUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.LoggerService;
import com.scbfkj.uni.service.SecurityService;
import com.scbfkj.uni.system.Config;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;
import java.util.stream.Stream;

@Component
@Aspect
public class LogAop {


    @Resource
    private SecurityService securityService;


    private static Map<String, RateLimiter> rateLimiterMap=new HashMap<>();


    @Around(value = "within( com.scbfkj.uni.api.*Api)")
    public ResponseEntity<Map<String, Object>> invokeAround(ProceedingJoinPoint joinPoint) {
        LocalDateTime requestTime = LocalDateTime.now();

        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//            前端需求添加header
//        if (requestAttributes.getResponse() != null) {
//            requestAttributes.getResponse().addHeader("Set-Cookie", "SameSite=None");
//        }
        HttpServletRequest request = requestAttributes.getRequest();

//        请求
        String uri = request.getRequestURI();


//        请求参数
        Object[] args = joinPoint.getArgs();
        Object returnData = null;
        String message = null;

        try {
            List<Map<String, Object>> ratelimitruleList = DataBase.query(Config.securityConnectionStr, "select * from ratelimitrule where 1=?", Collections.singletonList(new Object[]{1}));
            Optional<Map<String, Object>> optional = ratelimitruleList.stream().filter(it -> {
                Object pathMatch = it.get("pathmatch");
                if (Objects.isNull(pathMatch)) return true;
                return uri.matches(pathMatch.toString());
            }).findFirst();
            if (optional.isPresent()) {
                Map<String, Object> map = optional.get();
                String pathMatch = map.get("pathmatch").toString();
                if(!rateLimiterMap.containsKey(pathMatch)){
                    String duration = map.getOrDefault("duration", 1).toString();
                    String limitValue = map.getOrDefault("limitvalue", 100).toString();
                    rateLimiterMap.put(pathMatch,RateLimiter.create(Double.parseDouble(limitValue),Integer.parseInt(duration), TimeUnit.SECONDS));
                }
                RateLimiter rateLimiter = rateLimiterMap.get(pathMatch);
                String timeOut = map.getOrDefault("timeout", 1).toString();
                boolean acquire = rateLimiter.tryAcquire(Integer.parseInt(timeOut), TimeUnit.SECONDS);
                if(!acquire){
                    message = map.getOrDefault("returnmessage", "请求频率过高，请降低请求频率").toString();
                    return ResponseEntity.ok(UniReturnUtil.fail(message));
                }
            }


            if (Config.securityEnable) {
                 List<Map<String,Object>>   apiInfos = DataBase.query(Config.securityConnectionStr, "select * from apiinfo", new ArrayList<>() {{
                        add(new Object[]{});
                    }});


                Optional<Map<String, Object>> requestpath = apiInfos.stream().filter(it -> {
                    Object o = it.get("requestpath");
                    return Objects.equals(o, uri);
                }).findAny();
                if (requestpath.isPresent()) {
                    Map<String, Object> stringObjectMap = requestpath.get();
                    Object o = stringObjectMap.get("securitykey");
                    if (Objects.nonNull(o)) {
                        String[] securityCheck = o.toString().split(",");
                        for (String s : securityCheck) {
                            if (Objects.equals(s, "token")) {
                                String appToken = RequestUtil.getAppToken();
                                if (Objects.isNull(appToken)) {
                                    return ResponseEntity.ok(UniReturnUtil.fail("没有找到token"));
                                }
                                Map<String, Object> result = securityService.verifyToken(appToken);
                                if (!result.get("code").equals("0")) {
                                    return ResponseEntity.ok(result);
                                }
                            }
                            if (Objects.equals("usertoken", s)) {
                                String userToken = RequestUtil.getUserToken();
                                if (Objects.isNull(userToken)) {
                                    return ResponseEntity.ok(UniReturnUtil.fail("没有找到 usertoken"));
                                }
                                Map<String, Object> result = securityService.checkUserToken(userToken);
                                if (!result.get("code").equals("0")) {
                                    return ResponseEntity.ok(result);
                                }
                            }
                        }
                    }
                }
            }
            returnData = joinPoint.proceed(args);
            return ((ResponseEntity<Map<String, Object>>) returnData);
        } catch (Throwable e) {
            if (Config.debug) {
                e.printStackTrace();
            }
//            错误异常消息
            message = UniReturnUtil.getMessage(e);
            return ResponseEntity.ok(UniReturnUtil.fail(message));
        } finally {
            Object finalReturnData = returnData;
            LoggerService.log(LoggerService.LogType.INTERFACE, new HashMap<>() {{
                put("requesttime", requestTime);
                put("requestpath", uri);
                put("requestdata", DataFormatUtil.toString(args));
                put("sessionid", RequestUtil.getSessionId());
                put("returndata", DataFormatUtil.toString(finalReturnData));
                try {
                    put("applicationid", RequestUtil.getAppId());
                } catch (Exception e) {
                        System.out.println(UniReturnUtil.getMessage(e));
                }
            }});
        }
    }
}
