package com.scbfkj.uni.service;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import com.zaxxer.hikari.pool.HikariPool;

import java.io.File;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class LoggerService {

    public enum LogType {

        USER("userlog"),
        INTERFACE("interfacelog"),
        SYSTEM("systemerrlog"),
        SERVICE("servicelog"),
        SERVICE_ERR("serviceerrlog");

        private String name;

        LogType(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

    }

    private static final String connection = """
            {
              "jdbcUrl": "jdbc:sqlite:%s",
              "driverClassName": "org.sqlite.JDBC",
              "enableCache":false
            }
            """;
    private static String currentFileName;

    private final static Set<Long> fileNames = new HashSet<>();
    private static final String dir = "logs/";

    public static void log(LogType type, Map<String, Object> data) {
//        if (true) return;
        long filename = System.currentTimeMillis() / Config.splitCount;
        currentFileName = filename + ".sqlite";
        String filePath = dir + currentFileName;
        String connectionStr = connection.formatted(filePath);
        synchronized (fileNames) {
            if (!fileNames.contains(filename)) {
                if (!new File(filePath).exists()) {
                    try {
                        DataBase.exec(connectionStr, """
                                create table logs
                                (
                                    logid         Integer
                                        primary key autoincrement,
                                    target     TEXT,
                                    currentfile TEXT,
                                    datasourceid     TEXT,
                                    expression     TEXT,
                                    datacontent      TEXT
                                )""");
                    } catch (Exception e) {
                        System.out.println(UniReturnUtil.getMessage(e));
                    }
                }
                fileNames.add(filename);
                fileNames.removeIf(it -> it + 1000 < filename);
            }
        }
        try {
            List<Object[]> datas = new ArrayList<>();
//            List<Map<String, Object>> targets = Config.targets.stream().filter(it -> Objects.equals(it.get(""), type.getName())).toList();
            List<Map<String, Object>> targets = DataBase.query(Config.centerConnectionStr, "select * from systeminfo where keyname=?", Collections.singletonList(new Object[]{type.getName()}));
            for (Map<String, Object> targetconnection : targets) {
                datas.add(new Object[]{type.getName(), filePath, targetconnection.get("datasourceid"), targetconnection.get("expression"), DataFormatUtil.toString(data)});
            }
            DataBase.updateBatch(connectionStr, "insert into logs (target,currentfile,datasourceid,expression,datacontent) values(?,?,?,?,?)", datas);
        } catch (Exception e) {
            System.out.println(UniReturnUtil.getMessage(e));
        }
    }

    public static void sendMessage() {
//        判断目录

        File file = new File(dir);

//        日志目录下的文件
        List<File> files = Arrays.stream(file.listFiles()).toList();
//        过滤出当前不使用的文件
        List<File> logsFiles = files.parallelStream()
                .filter(logFile -> {
                    String fileName = logFile.getName();
                    return !fileName.equals(currentFileName) && fileName.endsWith("sqlite");
                }).toList();
//        记录出错的日志文件
        List<String> errorFileNames = new ArrayList<>();
        logsFiles.parallelStream()
                .map(logFile -> {
                    String fileName = logFile.getName();
//                    转成连接字符串
                    return connection.formatted(dir + fileName);
                })
                .flatMap(connectionStr -> {
//                    查询数据
                    try {
                        return DataBase.query(connectionStr, "select * from logs where 1=?", Collections.singletonList(new Object[]{1})).stream();
                    } catch (Exception e) {
                        Matcher matcher = Pattern.compile("logs/\\d+\\.sqlite").matcher(connectionStr);
                        if (matcher.find()) {
                            errorFileNames.add(matcher.group());
                        }
                        return Stream.empty();
                    }
                })
//                分组
                .collect(Collectors.groupingBy((log) -> log.get("target").toString()))
                .entrySet().parallelStream().forEach(stringListEntry -> {
                    String targetName = stringListEntry.getKey();
//                    需要发送的数据
                    List<Map<String, Object>> value = stringListEntry.getValue();
//                    按照目标连接字符串分组
                    Map<String, List<Map<String, Object>>> targetconnection = value.stream().collect(Collectors.groupingBy(data -> data.get("datasourceid").toString()));

                    targetconnection.forEach((datasourceid, value1) -> {
//                        获取发送的数据流
                        Stream<String> datacontentStream = value1.stream().map(data -> data.get("datacontent").toString());

//                        解析发送目标连接字符串
                        try {
                            List<Map<String, Object>> dataSourceList = DataBase.query(Config.centerConnectionStr, "select * from datasource where datasourceid=?", Collections.singletonList(new Object[]{datasourceid}));
                            if (dataSourceList.isEmpty()) {
                                return;
                            }
                            Map<String, Object> config = dataSourceList.get(0);
                            String type = config.getOrDefault("datasourcetype", "DB").toString();
                            String connectionStr = config.get("connectset").toString();
                            String expression = value1.get(0).get("expression").toString();
                            List<Object> parameters = new ArrayList<>();

                            switch (type.toUpperCase()) {
                                case "ES", "KAFKA" -> {
                                    parameters.add(DataFormatUtil.toMap(connectionStr));
                                    parameters.add(expression);
                                    parameters.add(connectionStr);
                                    parameters.add(targetName);
                                    parameters.add(datacontentStream.toList());
                                    DataProcessService.processByAlgorithm("1", parameters);
                                }
                                case "DB" -> {
                                    parameters.add(connectionStr);
                                    parameters.add(targetName);
                                    parameters.add(datacontentStream.map(DataFormatUtil::toMap).map(dataContent -> ((Map<String, Object>) dataContent)).toList());
                                    parameters.add("1");
                                    parameters.add(null);
                                    parameters.add(null);
                                    DataProcessService.processByAlgorithm("3", parameters);
                                }
                            }
                        } catch (Exception e) {
//                        当前数据是在哪一个数据库文件中
                            String currentfile = value.stream().findFirst().map(d -> d.get("currentfile").toString()).orElse(null);
                            errorFileNames.add(currentfile);
                            System.out.println(UniReturnUtil.getMessage(e));
                        }
                    });
                });
        logsFiles.stream().filter(f ->
                errorFileNames.stream().filter(Objects::nonNull).noneMatch(errorFileName ->
                        errorFileName.contains(f.getName()))
        ).forEach(f -> {
            try {
                String connectionStr = connection.formatted(dir + f.getName());
                DataBase.exec(connectionStr, "delete  from logs");
                HikariPool hikariPool = DataBase.dataSourcePools.remove(connectionStr);
                if (Objects.nonNull(hikariPool))
                    hikariPool.shutdown();
//                删除文件一直不成功
                f.delete();
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

    }
}
