package com.scbfkj.uni.system;

import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.ControlService;
import com.scbfkj.uni.service.LoggerService;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;

import java.io.*;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;


@Component
public class SystemInit {


    @Value("${db.center.config}")
    private String centerConfig;

    @Value("${db.security.config}")
    private String securityConfig;

    @Value("${app.security.enable:true}")
    private boolean securityEnable = false;
    @Value("${app.container.code}")
    private String containerCode;


    @Value("${debug:false}")
    private boolean debug = true;

    @PostConstruct
    public void init() throws Exception {
        Config.containerCode = containerCode;
        Config.securityEnable = securityEnable;
        Config.debug = debug;
        Config.centerConnectionStr = DataEncryptionUtil.decryptRSAByPrivateKey(centerConfig);
        Config.securityConnectionStr = DataEncryptionUtil.decryptRSAByPrivateKey(securityConfig);

//        日志配置初始化
        Config.targets.addAll(DataBase.query(Config.centerConnectionStr, "select * from systeminfo where 1=?", Collections.singletonList(new Object[]{1})));

        initializeSystemEnvironment();
        ControlService.startServiceByContainerCode();
//        日志任务
        ScheduleUtil.start(LoggerService::sendMessage, "*/1 * * * * *");
//        LoggerService.logSystemError(LocalDateTime.now(), "", "");
        LoggerService.log(LoggerService.LogType.SYSTEM, new HashMap<>() {{
            put("occurrencetime", LocalDateTime.now());
        }});
    }

    private void initializeSystemEnvironment() {
//        运行环境初始化
        File resource = new File("logs");
        boolean exists = resource.exists();
        if (!exists) {
            if (!resource.mkdirs()) {
                throw new RuntimeException("创建日志目录失败");
            } else {
                System.out.println("创建目录成功:" + resource.getAbsolutePath());
            }
        }
    }

    private String readConfigFile() throws IOException {
        File resource = new File("config");
        boolean exists = resource.exists();
        if (exists) {
            BufferedInputStream inputStream = new BufferedInputStream(new FileInputStream(resource));
            byte[] bytes = new byte[1024];
            StringBuilder builder = new StringBuilder();
            while (inputStream.read(bytes) > 0) {
                builder.append(new String(bytes));
            }
            return builder.toString();
        } else {
            return "{}";
        }
    }

}
